#include <asf.h>
#include <string.h>

#include "conf_fatfs.h"


/** Size of the file to write/read. */
#define DATA_SIZE 1024

/** Test settings: Number of bytes to test */
#define TEST_SIZE   (DATA_SIZE * 1000)


typedef struct _f_list_
{
	char ksname[_MAX_LFN + 1];
}	File_List;


static uint32_t Test_FileRead(uint32_t);
static uint32_t Test_FileWrite(uint32_t);


/* Read/write buffer */
static uint8_t data_buf[DATA_SIZE];
static uint8_t comp_buf[DATA_SIZE];

static File_List s_file_list[50];

static uint8_t s_file_list_no = 0;

/* IMPORTANT!!!
* Declare a FIL variable as static to avoid stack usage.
* They each contain an array of maximum sector size. */
static FIL s_file_object;




/**
 * \brief Scan files under a certain path.
 *
 * \param path Folder path.
 *
 * \return Scan result, 1: success.
 */
#if 0
static FRESULT Scan_Files(char *path, File_List *pfile_list)
{
	FRESULT res;
	FILINFO fno;
	DIR dir;
	int32_t i;
	char *pc_fn;

	/* Open the directory */
	res = f_opendir(&dir, path);
	if (res == FR_OK) {
		i = strlen(path);
		for (;;) {
			res = f_readdir(&dir, &fno);
			if (res != FR_OK || fno.fname[0] == 0) {
				break;
			}

			pc_fn = fno.fname;

			if (*pc_fn == '.') {
				continue;
			}

			/* Check if it is a directory type */
			if (fno.fattrib & AM_DIR) {
				sprintf(&path[i], "/%s", pc_fn);
				res = Scan_Files(path, pfile_list);
				if (res != FR_OK) {
					break;
				}

				path[i] = 0;
			} else {
				printf("%s/%s\r\n", path, pc_fn);
			}
		}
	}

	return res;
}
#else
static FRESULT Scan_Files(char *path, File_List *pfile_list)
{
	FRESULT res;
	FILINFO fno;
	DIR dir;

	char *pc_fn;
	int32_t i;


	/* Open the directory */
	if((res = f_opendir(&dir, path)) == FR_OK)
	{
		i = strlen(path);

		for (;;)
		{
			res = f_readdir(&dir, &fno);
			if (res != FR_OK || fno.fname[0] == 0)
			{
				break;
			}

			pc_fn = fno.fname;

			if (*pc_fn == '.')
			{
				continue;
			}

			/* Check if it is a directory type */
			if (fno.fattrib & AM_DIR)
			{
				sprintf(&path[i], "/%s", pc_fn);

				res = Scan_Files(path, pfile_list);
				if (res != FR_OK)
				{
					printf("Scan_Files() ERROR\r\n");
					break;
				}

				path[i] = 0;
			}
			else
			{
				char *dest_ptr;
				char *ptr;

				ptr = (char *)path;
				dest_ptr = (char *)(pfile_list[s_file_list_no].ksname);

				while(*ptr != NULL)
				{
					*dest_ptr++ = *ptr++;
				}

				*dest_ptr++= '/';
				ptr = (char *)fno.fname;

				while(*ptr != NULL)
				{
					*dest_ptr++ = *ptr++;
				}

				*dest_ptr = '\0';

				s_file_list_no++;
			}
		}
	}
	else
	{
		printf("[L%d] f_opendir() ERROR 0x%X\r\n", __LINE__, res);
	}

	return res;
}
#endif




/* brief Configure the Console UART. */

static void configure_console(void)
{
	const usart_serial_options_t usart_serial_options = {
		.baudrate = 115200,
		.charlength = US_MR_CHRL_8_BIT,
		.paritytype = US_MR_PAR_NO,
		.stopbits = US_MR_NBSTOP_1_BIT,
	};

	/* Configure console UART. */
	sysclk_enable_peripheral_clock(CONSOLE_UART_ID);
	stdio_serial_init(CONSOLE_UART, &usart_serial_options);
}




/**
 *  \brief Application entry point for FatFS example.
 */
int main(void)
{
	FRESULT res;
	uint32_t num_lun, disk_dev_num;
	Ctrl_status status;

	/* Initialize the system */
	sysclk_init();
	board_init();

	configure_console();

	/* Output example information */
	printf("[SAME70 XPLD - SD MMC FatFS Test]\r\n");
	printf("SAME70 - Processor Clock = %d Hz\r\n", sysclk_get_cpu_hz());
	printf("SAME70 - Peripheral Clock = %d Hz\r\n\n", sysclk_get_peripheral_hz());

	pmc_enable_periph_clk(SD_MMC_0_CD_PIO_ID);

	/* Initialize SD MMC stack */
	sd_mmc_init();

	while (1)
	{
		printf("Please plug an SD, MMC or SDIO card in slot.\r\n");

		do
		{
			if ((status = sd_mmc_test_unit_ready(0)) == CTRL_FAIL)
			{
				printf("Card install FAIL\r\n");
				printf("Please unplug and re-plug the card.\r\n");
				
				while (CTRL_NO_PRESENT != sd_mmc_check(0));
			}
		} while (CTRL_GOOD != status);

		break;
	}

	if((num_lun = get_nb_lun()) == 0)
	{
		printf("No Disk assigned\r\n");
		while(1);
	}

	printf("num_lun = %d\r\n", num_lun);

	disk_dev_num = num_lun - 1;

	res = Test_FileWrite(disk_dev_num);

	if (res == FR_OK)
	{
		printf("\r\n-I- DISK %d Test passed !\r\n\n", disk_dev_num);
	}
	else
	{
		printf("\r\n-F- DISK %d Test Failed !\r\n\n", disk_dev_num);
	}

	while (1);
}




static uint32_t Test_FileWrite(uint32_t disk_dev_num)
{
	/* IMPORTANT!!!
	* Declare a FATFS variable as static to avoid stack usage.
	* They each contain an array of maximum sector size. */
	static FATFS fs;	// f_read() error doesn't happen.
	//FATFS fs;			// f_read() error happens.

	FILINFO fno;
	DIR     dirs;
	FRESULT res;

	uint32_t file_size;
	uint32_t byte_read;
	uint32_t byte_written;

	char root_directory[2];
	TCHAR test_folder_name[5];
	TCHAR test_folder_location[8];

	/* File name to be validated */
	TCHAR file_name[20];

	uint32_t slice_size, slice_sum;
	uint32_t count;
	uint8_t num;

#if _FS_MINIMIZE != 0
	printf("Please run Full version FAT FS test\r\n");
	return 1;
#endif

#if _FS_READONLY != 0
	printf("Set _FS_READONLY to 0 to test file write\r\n");
	return 1;
#endif

	sprintf(root_directory, "%d:", disk_dev_num);
	sprintf(test_folder_name, "%s", (const char *)"TEST\0");
	sprintf(test_folder_location, "%s/%s", root_directory, test_folder_name);
	sprintf(file_name, "%s/Basic.bin", test_folder_location); /*File path*/

	/* Mount disk*/
	printf("-I- Mount disk %d\r\n", (int)disk_dev_num);

	/* Clear file system object */
	memset(&fs, 0, sizeof(FATFS));

	res = f_mount(&fs, "", 1);

	//res = FR_NO_FILESYSTEM;	// just for format test...
	printf("[L%d] res = %d\r\n", __LINE__, res);
	
	if (res == FR_NO_FILESYSTEM)
	{
		/* Format disk */
		printf("Format disk %d\r\n", (int)disk_dev_num);
		printf("Please wait a moment during formatting...\r\n");
		res = f_mkfs(root_directory,
					FM_EXFAT,
					//FM_ANY,
					0,
					data_buf,
					DATA_SIZE);

		if (res != FR_OK) {
			printf("-E- f_mkfs pb: 0x%X\r\n", res);
			return res;
		}

		printf("Disk format finished !\r\n");
	}
	else if (FR_OK != res)
	{
		printf("-E- f_mount pb: 0x%X\r\n", res);
		return res;
	}

	/*Create a directory*/

	if(strcmp(test_folder_name, "TEST\0") == 0)
	{
		puts("-I- Creating test directory !\r");
		res = f_mkdir(test_folder_name);
		//if (res != FR_OK) {
		if ((res != FR_OK) && (res != FR_EXIST)) {
			printf("-E- f_mkdir pb: 0x%X\r\n", res);
			return res;
		}
	}

	/*Opening the directory*/
	puts("-I- Opening directory !\r");

	res = f_opendir(&dirs, root_directory);
	if (res == FR_OK) {
		/* Erase sd card to reformat it ? */
		puts("-I- The disk is already formatted.\r");

		/* Display the file tree */
		puts("-I- Display files contained in the memory :\r");
		strcpy((char *)data_buf, root_directory);

		s_file_list_no = 0;
		Scan_Files((char *)data_buf, s_file_list);
	}
	else
	{
		printf("-E- f_opendir pb: 0x%X\r\n", res);
		return res;
	}

	/*Changing  the directory*/
	puts("-I- Changing to test directory !\r");
	res = f_chdir(test_folder_location);

	if (res != FR_OK) {
		printf("-E- f_chdir pb: 0x%X\r\n", res);
		return res;
	}

	puts("-I- Changed to directory\r");

#if FF_FS_EXFAT == 0
	/*Get the current working directory*/
	puts("-I- Getting current working directory !\r");
	res = f_getcwd(get_test_folder, 8);
	if (res != FR_OK) {
		printf("-E- getcwd not working\r\n");
		return res;
	}

	res = strcmp(test_folder_location, get_test_folder);

	printf("%s  %s\r\n", test_folder_location, get_test_folder);

	if (res != 0) {
		printf("-E- Mismatch in f_getcwd and actual Folder name\r\n");

		return res;
	}
#endif

#if FF_USE_FIND == 1
	/*Searching a file in location*/
	res = f_findfirst(&dirs, &fno, "", "Basic.bin"); /* Start to search for bin files */
	if (res != FR_OK) {
		printf("-E- f_findfirst pb: 0x%X\r\n", res);
		return res;
	}

	if(memcmp(fno.fname, "Basic.bin", 9) == 0)
	{
		printf("-I file name search found \"%s\"\r\n", fno.fname); /* Display the item name */
		printf("## File exists. Delete file... ##\r\n");

		puts("-I- Delete file\r");
		res = f_unlink(fno.fname);
		if (res != FR_OK) {
			printf("-E- f_unlink pb: 0x%X\r\n", res);
			return res;
		}
	}
#endif /* FF_USE_FIND */

	/* Create a new file */
	printf("Create a file : \"%s\"\r\n", file_name);

	if ((res = f_open(&s_file_object, (char const *)file_name, FA_CREATE_NEW | FA_WRITE)) != FR_OK)
	{
		printf("-E- f_open create pb: 0x%X\r\n", res);
		return res;
	}

	/* Write pattern data to the buffer */
	for (count = 0; count < DATA_SIZE; count++)
		data_buf[count] = (count % 256);

	printf("Write file\r\n");

	for (count = 0; count < TEST_SIZE; count += DATA_SIZE)
	{
		if((byte_written = File_Write((void *)data_buf, sizeof(U8), DATA_SIZE, &s_file_object)) != DATA_SIZE)
		{
			printf("-E- File_Write pb: 0x%X\r\n", res);
			return 1;
		}

		printf("[%d] %d\r", count, byte_written);
	}

	printf("\r\nWriting %d done.\r\n", count);

	/* Flush after writing */
	puts("-I- Synching file\r");

	if ((res = f_sync(&s_file_object)) != FR_OK)
	{
		printf("-E- f_sync pb: 0x%X\r\n", res);
		return res;
	}

	/* Close the file */
	printf("Close file\r\n");

	if ((res = f_close(&s_file_object)) != FR_OK)
	{
		printf("-E- f_close pb: 0x%X\r\n", res);
		return res;
	}

	/* Open the file */
	printf("Open the same file : \"%s\"\r\n", file_name);

	if ((res = f_open(&s_file_object, (char const *)file_name, FA_OPEN_EXISTING | FA_READ)) != FR_OK) {
		printf("-E- f_open read pb: 0x%X\r\n", res);
		return res;
	}

	/* Read file */
	file_size = s_file_object.obj.objsize;

	/* Write a comparing pattern to the buffer */
	for (count = 0; count < DATA_SIZE; count++)
		comp_buf[count] = (count % 256);

	printf("Read file -- file_size = %d\r\n", file_size);

	slice_size = DATA_SIZE;
	slice_sum = 0;

	while(slice_sum < file_size)
	{
		if(slice_size > (file_size - slice_sum))
			slice_size = file_size - slice_sum;

		//printf("[Loop %d] Read file... ", slice_sum / DATA_SIZE);

		if ((byte_read = File_Read((void *)data_buf, sizeof(U8), slice_size, &s_file_object)) != slice_size)
		{
			printf("-E- File_Read pb: 0x%X\r\n", res);
			return 1;
		}

		if(memcmp(comp_buf, data_buf, slice_size) == 0)
		{
			printf("Matched\r");
			memset(data_buf, 0, slice_size);
		}
		else
		{
			printf("Not Matched\r\n");

			for(uint32_t count = 0; count < slice_size; count++)
			{
				if(comp_buf[count] != data_buf[count])
					printf("[%d] 0x%X 0x%X\r\n", count, comp_buf[count], data_buf[count]);
			}

			f_close(&s_file_object);
			while(1);
		}
		
		slice_sum += slice_size;
	}

	printf("\r\nReading %d done.\r\n", slice_sum);

	/* Close the file*/
	printf("\r\nClose file...\r\n");

	if ((res = f_close(&s_file_object)) != FR_OK)
	{
		printf("-E- f_close pb: 0x%X\r\n", res);
		return res;
	}

	/* Close directory */
	if ((res = f_closedir(&dirs)) != FR_OK)
	{
		printf("-E- f_closedir pb: 0x%X\r\n", res);
		return res;
	}

	/* Compare the read data with the expected data */
	printf("File Write Test Ok\r\n");


	return 0;
}

/* End of File */
